(** Easing *)

let linear t = t

let quad_in t = t *. t
let quad_out t = t *. (2.0 -. t)
let quad_inOut t =
  if t < 0.5 then 2.0 *. t *. t
  else (4.0 -. 2.0 *. t) *. t -. 1.0

let cubic_in t = t *. t *. t
let cubic_out t =
  let t = t -. 1.0 in
  t *. t *. t +. 1.0

let cubic_inOut t =
  if t < 0.5 then 4.0 *. t *. t *. t
  else
    let t = t *. 2.0 in
    let t = t -. 2.0 in
    (t *. t *. t +. 2.0) *. 0.5

let quart_in t = t *. t *. t *. t
let quart_out t =
  let t = t -. 1.0 in
  1.0 -. t *. t *. t *. t

let quart_inOut t =
  let t = t *. 2.0 in
  if 1.0 > t
  then 0.5 *. t *. t *. t *. t
  else let t = t -. 2.0 in
    -0.5 *. (t *. t *. t *. t -. 2.0)

type ease =
  | Linear
  | QuadIn
  | QuadOut
  | QuadInOut
  | CubicIn
  | CubicOut
  | CubicInOut
  | QuartIn
  | QuartOut
  | QuartInOut

let f e t =
  match e with
  | Linear     -> linear t
  | QuadIn     -> quad_in t
  | QuadOut    -> quad_out t
  | QuadInOut  -> quad_inOut t
  | CubicIn    -> cubic_in t
  | CubicOut   -> cubic_out t
  | CubicInOut -> cubic_inOut t
  | QuartIn    -> quart_in t
  | QuartOut   -> quart_out t
  | QuartInOut -> quart_inOut t
