(** Timeline *)

(* timeline types *)

type time = float  (* in seconds *)

type 'a animated = At of time * 'a | Change of time * time * 'a * 'a

type 'a timed = Static of 'a | Animated of 'a animated list

type float1 = float timed
type float2 = (float * float) timed
type float3 = (float * float * float) timed
type float4 = (float * float * float * float) timed


(* timeline functions *)

let inter1 t t1 t2 v1 v2 =
  let m = (t -. t1) /. (t2 -. t1) in
  ( v1 +. (m *. (v2 -. v1)) )

let inter2 t t1 t2 (a1,b1) (a2,b2) =
  let m = (t -. t1) /. (t2 -. t1) in
  ( a1 +. (m *. (a2 -. a1)),
    b1 +. (m *. (b2 -. b1)) )

let inter3 t t1 t2 (a1,b1,c1) (a2,b2,c2) =
  let m = (t -. t1) /. (t2 -. t1) in
  ( a1 +. (m *. (a2 -. a1)),
    b1 +. (m *. (b2 -. b1)),
    c1 +. (m *. (c2 -. c1)) )

let inter4 t t1 t2 (a1,b1,c1,d1) (a2,b2,c2,d2) =
  let m = (t -. t1) /. (t2 -. t1) in
  ( a1 +. (m *. (a2 -. a1)),
    b1 +. (m *. (b2 -. b1)),
    c1 +. (m *. (c2 -. c1)),
    d1 +. (m *. (d2 -. d1)) )


let rec val_at inter t = function
  | At(t1, v) :: At(t2,_) :: _
  | At(t1, v) :: Change(t2,_,_,_) :: _
    when t1 <= t && t < t2 -> (v)
  | At(t, v) :: [] -> (v)
  | Change(_,t2,_,v2) :: []
    when t >= t2 -> (v2)
  | Change(t1,t2,v1,v2) :: _
    when t1 <= t && t <= t2 -> inter t t1 t2 v1 v2
  | _ :: tl -> val_at inter t tl
  | [] -> invalid_arg "val_at"

let get_val inter t = function
  | Static v -> v
  | Animated anim -> val_at inter t anim

let get_val1 = get_val inter1 ;;
let get_val2 = get_val inter2 ;;
let get_val3 = get_val inter3 ;;
let get_val4 = get_val inter4 ;;

